{{
  Lapidary Saw Controller with Telnet Status
  Roger Williams
  October, 2010
}}
con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  'pin cross reference indeces
  #0
  runsw
  limit
  slack
  rot
  unused        'has pullup, no fn
  future        'available, no pullup
  '
  drive         '1st output
  saw
  npins

  ninputs = future

  video = 12    'PropRPM standard

  'states codes
  #0
  idle
  running
  rot_timeout
  pwr_on_running
  slack_timeout
  at_limit

  slack_err_seconds=60

  led_base_pin=16
  led_flash_pin=25

  'ENC28J60 pins
  nic_cs      = 3
  nic_sck     = 4
  nic_si      = 5
  nic_so      = 6
  nic_int     = 7
  nic_xtalout = -1

obj

  text  : "tv_text"
  serv  : "api_telnet_serial"  'telnet server
  
dat

' The 7 i/o pins are dereferenced to a contiguous set, so
' common functions can be easily handled by common code.

inpinxref     byte 27, 26, 11, 10, 9, 8, 1, 0

' The PropRPM LED bar is used to display pin IO states.
' The 8th LED flashes an error code according to an
' appropriate pattern.

err_patt      word      %00000000_00000000     'no error
              word      %11001100_11001100     'fast blink = running
              word      %11101110_10000000     'D=drive failed
              word      %10111011_10100000     'P=power on running
              word      %10101000_10101000     'SS=slack timeout
              word      %11111111_00000000     'slow blink=end (at limit running)

' Ethernet sstuff from  cl_serv_ex
' Yes it's bad form to hardcode this stuff but it's a personal project, I can
' always re-blow the firmware if I have to change anything

mac_addr      byte    $10, $00, $00, $00, $00, $01       ' propeller's mac address

ip_addr       byte    192, 168, 23, 42                   ' propeller's ip address
ip_subnet     byte    255, 255, 255, 0                   ' network subnet
' these aren't used in this application but the driver expects them
ip_gateway    byte    192, 168, 23, 1                    ' network gateway (router)
ip_dns        byte    192, 168, 23, 1                    ' network dns

port_host     byte    23                                 ' propeller server host port = telnet

var

  long  sec_interval

  byte  pin[npins]
  byte  pinstate[npins]
  long  pincnt[npins]

  byte  nstate

  long  dbinterval
  
  long  ebmask
  long  ebcnt
  long  ebinterval

  long  slackcnt
  byte  slack_err_cnt

  long  rotcnt
  byte  rot_sec_cnt
  byte  rotstate
  byte  rotstart
  byte  rotstopped
  long  rotposition
  
  byte  mstate

  long  run_time_cnt
  byte  run_sec
  byte  run_min

  byte  numbuf[12]      'used to format fixed-width decimal strings
  byte  descbuf[120]    'actually 2 buffers, one used while other written
  byte  lasttelmsg[60]  'used only to overwrite telnet activity character on LF
  word  wbase
  word  ubase

  byte  actchar
  byte  limit_cmd
    
  long  myServerIP               ' remote server's IP, packed in a 32 bit number
  word  myServerPort             ' remote server port number
  word  myHostPort               ' our server host port number

  long  ServStack[100]

  long  servcnt
  byte  servlf

pub main

  text.start(video)
  serv.start(nic_cs, nic_sck, nic_si, nic_so, nic_int,-1,@mac_addr,@ip_addr)          
  delay_ms(150)                 'let nic finish initializing

  myHostPort := byte[@port_host]

  cognew(MyServer, @ServStack)
  delay_ms(150)                 'wait for server cog to start

  'init pin xref array
  bytemove(@pin,@inpinxref,npins)

  dbinterval    := clkfreq / 20
  ebinterval    := clkfreq / 5
  sec_interval  := clkfreq
  slack_err_cnt := 0
  
  'set output pins
  dira[pin[saw]]~~
  dira[pin[drive]]~~
  outa[led_base_pin..led_flash_pin]~~
  dira[led_base_pin..led_flash_pin]~~

  wbase := @descbuf
  ubase := wbase+60

  actchar := "-"
  
  'initialize the input debouncer
  start_db
  'and the rotation detector
  start_rot

  'if we powered up running it's an error,
  'otherwise go to idle
  '
  mstate := 255 'force change to trigger init
  if pinstate[runsw]
    setstate(pwr_on_running)
  else
    setstate(idle)

  'main program loop
  repeat

    debounce
    advance_rot
    advance_state
    display

pri start_db | p
  repeat p from 0 to ninputs-1
    pinstate[p] := !ina[pin[p]]  & 1
    pincnt[p] := cnt

pri debounce | p
  '
  'common code to read and debounce all inputs
  'this sets the pinstate entry to 0 or 1,
  'inverted to provide positive logic from
  'the pull-up inputs
  '
  repeat p from 0 to ninputs-1
    nstate := !ina[pin[p]]  & 1
    'text.bin(nstate,1)
    if nstate == pinstate[p]
      pincnt[p] := cnt
    elseif (cnt - pincnt[p]) > dbinterval
      'new state has persisted
      pinstate[p] := nstate
    'text.bin(pinstate[p],1)
    'text.out(32)
{ text.dec(mstate)
  text.out(32)
  text.hex(rotposition,3)
  text.out(32)
  text.hex(rotstart,1)
  text.out(32)
  text.hex(rotstopped,1)
  text.out(32)
  text.dec(slack_err_cnt)
  text.out(13) }
  
pri start_rot
  '
  'set the rotation engine on assumption that we
  'are normally starting an idled system; this
  'inhibits the timeout error and use of the
  'first 'period' from setting the cnt at an
  'unknown point in the rotation
  '
  rotstart := true
  rotstopped :=false
  rotcnt := cnt
  rot_sec_cnt := 0
  rotstate := pinstate[rot]

pri advance_rot
  '
  'check rotation sensor, advance position, and
  'set timeout as appropriate; rotation senseor
  'is active in all states
  '
  'limit switch always resets position
  '
  if pinstate[limit]
    if rotposition <> 0
      rotposition := 0
      show_desc
    
  'update period count
  '
  if not rotstopped
    if (cnt - rotcnt) > sec_interval
      rotcnt := cnt
      rot_sec_cnt++

  if pinstate[rot] == rotstate
    'no new pulses
    if (not rotstopped) or rotstart
      if rot_sec_cnt > 30
        rotstopped := true
        rotstart := false
    return

  'the pin state did change
  rotstate := pinstate[rot]
  if rotstate == 0
    return 'we only act on 0 to 1
  '
  '0 to 1 transition, let's calculate
  '
  if rotstopped or rotstart
    'this is first pip after stoppage
    'so reset everything & prepare to rot
    rotcnt := cnt
    rot_sec_cnt := 0
    rotstopped := false
    rotstart := false
    return

  if rot_sec_cnt > 5
    'long / slow rotation is forward
    rotposition++
  else
    'fast rotation is backward/reset
    rotposition--
  rot_sec_cnt := 0
  rotcnt := cnt
  show_desc

pri advance_state
  '
  'perform state transitions as appropriate
  '
  'losing run switch trumps all
  '
  if pinstate[runsw] == 0
    setstate(idle)
    return

  case mstate
    idle:
      if pinstate[runsw]
        start_rot
        slack_err_cnt := 0
        setstate(running)
    running, rot_timeout:
      'running and rot_timeout states are the same
      'except for displaying rot_timeout error
      if rotstopped
        setstate(rot_timeout)
      else
        setstate(running)
      '
      'advance run time
      '
      if (cnt - run_time_cnt) > sec_interval
        run_time_cnt := cnt
        run_sec++
        if run_sec > 59
          run_sec := 0
          run_min++
        show_desc
      '
      'If there's slack we cut the drive and
      'continue running the saw, unless the slack
      'persists for too long in which case we
      'error out
      '
      if pinstate[slack]
        outa[pin[drive]] := 0
        start_rot 'inhibit rot_timeout error since drive stopped
        if (cnt - slackcnt) > sec_interval
          slackcnt := cnt
          slack_err_cnt++
          if slack_err_cnt > slack_err_seconds
            setstate(slack_timeout)
            return
      else
        outa[pin[drive]] := 1
        slackcnt := cnt
        slack_err_cnt := 0
      '
      'If we hit the limit, we're done
      if pinstate[limit] or limit_cmd
        limit_cmd := false
        setstate(at_limit)
          
pri setstate(newstate)

  if newstate == mstate
    return

  'state has actually changed
  'reset flash msg bit cursor
  ebmask := 0
  ebcnt := cnt
  'and linefeed the text displays
  desc_newline

  mstate := newstate
  case mstate
    idle:
      'saw can run, drive is off
      outa[pin[saw]]~~
      outa[pin[drive]]~
    running, rot_timeout:
      'saw and drive running; rot_timeout keeps
      'running but displays error
      outa[pin[saw]]~~
      outa[pin[drive]]~~
      run_sec := 0
      run_min := 0
      run_time_cnt := cnt
      limit_cmd := false
    pwr_on_running, slack_timeout, at_limit:
      'saw and drive are off
      outa[pin[saw]]~
      outa[pin[drive]]~
  'display the resulting change
  show_desc

pri display | i, p

  'update the LED bar with pin states and flash state/error msg
  'output pin states aren't updated by debounce
  '
  pinstate[drive] := ina[pin[drive]]
  pinstate[saw] := ina[pin[saw]]
  '
  repeat i from 0 to npins-1
    outa[led_base_pin+i] := !pinstate[i]
      '
  'see if we're ready for next bit of err flash
  '
  if (cnt -  ebcnt) > ebinterval
    ebcnt := cnt
    if ebmask == 0
      ebmask := %10000000_00000000
    outa[led_flash_pin] := not (word[@err_patt + mstate << 1] & ebmask)
    ebmask >>= 1


pri desc_home
{
  Home the desc cursor to rebuild the line but do
  not advance to the next line.
}
  text.out(10)  'set X position
  text.out(0)   'to zero

pri desc_newline
  text.out(13)  'CR to video
  servlf := true
  
pri show_desc

  'Build the appropriate TV/Telnet text string for
  'our current state and status; this is generally
  'done once per second or when something changes
  'this homes the cursor to the left to potentially
  'overwrite an existing line but it doesn't go
  'to the next line (state changes trigger that)

  case mstate
    idle:
      bytemove(wbase, string("Idle @"), 7)
      descpos

    running:
      bytemove(wbase, string("RUN  @"), 7)
      descpos
      descrun

    rot_timeout:
      bytemove(wbase, string("DrvE @"), 7)
      descpos
      descrun

    pwr_on_running:
      bytemove(wbase, string("SHUTDOWN: Power On Running"),27)

    slack_timeout:
      bytemove(wbase, string("SHUTDOWN: Slack Timeout"),24)
         
    at_limit:
      bytemove(wbase, string("SHUTDOWN: Reached Limit"),24)
  
  case wbase
    @descbuf:
      wbase := @descbuf + 60
      ubase := @descbuf
    other:
      wbase := @descbuf
      ubase := @descbuf + 60
        
  desc_home
  text.str(ubase)
  
pri descpos
  if rotposition < 0
    byte[wbase+6] := "-"
  else
    byte[wbase+6] := "+"
  strdec(wbase+7,||rotposition,3)

pri descrun
  byte[wbase+10] := " "
  strdec(wbase+11,run_min,2)
  byte[wbase+13] := ":"
  zstrdec(wbase+14,run_sec,2)
  if pinstate[slack]
    bytemove(wbase+16, string(" SLACK"),7)
  else
    bytemove(wbase+16, string("      "),7)   

PRI strdec(loc,value,len) | j

  j := len
  byte [@numbuf+j] := 0
  j--
  repeat
    byte[@numbuf+j] := value // 10 + "0"
    j--
    value /= 10
    if value == 0 or j < 0
      quit
  '
  repeat while j => 0
    byte [@numbuf+j] := " "
    j--
  bytemove(loc,@numbuf,len+1)

pri zstrdec(loc,value,len) | j
  'zero padded version
  j := len
  byte [@numbuf+j] := 0
  j--
  repeat
    byte[@numbuf+j] := value // 10 + "0"
    j--
    value /= 10
    if value == 0 or j < 0
      quit
  '
  repeat while j => 0
    byte [@numbuf+j] := "0"
    j--
  bytemove(loc,@numbuf,len+1)



PUB MyServer | c, hithere
  ' Telnet server (runs in a seperate cog)
  \serv.listen(MyHostPort)
  repeat
    \serv.rxcheck
    \serv.resetBuffers
    hithere := true
    servcnt := cnt
    repeat while serv.isConnected
      c := \serv.rxcheck
      case c
        27:
          \serv.close
          quit
        "Z","z":
          rotposition := 0
        "L","l":
          limit_cmd := true
      if hithere
        serv.str(string("Telnet Lapidary Saw Monitor",13,10))
        serv.tx(13)
        hithere := false
      if (cnt - servcnt) > sec_interval
        'activity indicator in all states alerts to dead
        'but unclosed telnet connection
        case actchar
          "/":
            actchar := "-"
          "-":
            actchar := "\"
          "\":
            actchar := "|"
          "|":
            actchar := "/"
          other:
            actchar := "-"
        servcnt := cnt
        serv.tx(13)
        if servlf
          'reissue last message with a couple of spaces
          'to overwrite activity char if it's there
          serv.str(@lasttelmsg)
          serv.str(string("  ",13,10))
          servlf := false
        bytemove(@lasttelmsg,ubase,strsize(ubase)+1)    
        serv.str(@lasttelmsg)
        'only send actchar for states that don't clock
        case mstate
          idle, pwr_on_running, slack_timeout, at_limit:
            serv.tx(32)
            serv.tx(actchar)
        serv.tx(13)
        

PRI delay_ms(Duration)
  waitcnt(((clkfreq / 1_000 * Duration - 3932)) + cnt)
       